/* runcap - run program and capture its output
   Copyright (C) 2017 Sergey Poznyakoff

   Runcap is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3 of the License, or (at your
   option) any later version.

   Runcap is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with Runcap. If not, see <http://www.gnu.org/licenses/>. */

#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include "runcap.h"

int
runcap_seek(struct runcap *rc, int stream, off_t off, int whence)
{
	struct filecapture *fp;
	off_t cur;
	
	fp = runcap_filecapture(rc, stream);
	if (!fp)
		return -1;

	cur = runcap_tell(rc, stream);
	switch (whence) {
	case SEEK_CUR:
		off = cur;
		break;

	case SEEK_END:
		off = fp->fc_leng + off;
		break;

	case SEEK_SET:
		break;

	default:
		errno = EINVAL;
		return -1;
	}
	
	if (off < 0) {
		errno = EINVAL;
		return -1;
	}

	cur -= fp->fc_level;
	
	if (cur <= off && off <= cur + fp->fc_level) {
		fp->fc_linestart = off - cur;
	} else if (fp->fc_tmpfd != -1) {
		if (lseek(fp->fc_tmpfd, off, SEEK_SET) == -1)
			return -1;
		fp->fc_level = 0;
		fp->fc_linestart = 0;
	} else {
		errno = EINVAL;
		return -1;
	}
	return 0;
}

		
	
	
